import { Injectable } from '@angular/core'
import { HttpEvent, HttpHandler, HttpInterceptor, HttpRequest, HttpResponse, HttpResponseBase } from '@angular/common/http'
import { Observable, of, throwError } from 'rxjs'
import { AuthService } from './auth.service'

@Injectable()
export class AuthInterceptorService implements HttpInterceptor {
  constructor(
    private authService: AuthService,
  ) {
  }

  intercept(req: HttpRequest<any>, next: HttpHandler): Observable<HttpEvent<any>> {
    if (req.url.includes('healthy')) {
      return next.handle(req)
    }

    const session = this.authService.session
    if (session) {
      let token = session?.token

      if (!token.startsWith('Bearer')) {
        token = `Bearer ${token}`
      }

      let headers = req.headers
      if (session.dataPartionId) {
        headers = headers.set('data-partition-id', session.dataPartionId)
      }

      if (session.userId) {
        headers = headers.set('x-user-id', session.userId)
      }
      headers = headers.set('Authorization', token)

      return next.handle(req.clone({ headers }))
    }

    return next.handle(req)
  }
}
