import { Component, OnInit } from '@angular/core';
import { ActivatedRoute, Router } from '@angular/router'
import { FormBuilder  } from '@angular/forms';

import { AuthService, AuthSession } from '../auth.service'
import { PolicyService } from '../policy.service'

@Component({
  selector: 'app-lookup-policy',
  templateUrl: './lookup-policy.component.html',
  styleUrls: ['./lookup-policy.component.scss']
})
export class LookupPolicyComponent implements OnInit {
  public policy: any
  public policyId: string = '';
  public noPolicyMessage = 'No Policy Selected';
  public lookupForm = this.fb.group({
    policyId: [this.policyId],
  })
  public evaluateForm = this.fb.group({
    query: [''],
    includeAuth: [false],
  })

  constructor(
    private api: PolicyService,
    public authService: AuthService,
    private route: ActivatedRoute,
    private router: Router,
    private fb: FormBuilder,
  ) { }

  ngOnInit(): void {
    // if (this.authService.hasSession()) {
    //   this.api.getPolicy("osdu", "acls.rego").subscribe((result) => {
    //     this.policy = result
    //   })
    // }

    this.route.queryParamMap.subscribe((qs) => {
      this.policyId = qs.get('policyId') ?? ''

      if (this.policyId?.startsWith('osdu/partition')) {
        this.policyId = this.policyId.substring(this.policyId.lastIndexOf('/') + 1)
      }

      if (this.policyId) {
        this.lookupForm.setValue({ policyId: this.policyId })
        this.lookupForm.updateValueAndValidity()
        this.fetchPolicy()
      }
    })
  }

  editSupported(): boolean {
    console.log("editSupported")
    const policyId = this.lookupForm.value.policyId
    if (policyId != null) {
        console.log("starts with " + policyId)
        return policyId.startsWith('osdu/partition') 
    } else {
      return false
    }
  }

  onLookupPolicy() {
    const policyId = this.lookupForm.value.policyId
    if (policyId != null) {
      this.router.navigate(['/lookup'], { queryParams: { policyId } })
    }
  }

  fetchPolicy() {
    this.noPolicyMessage = 'Loading policy…';
    if (this.authService.hasSession() && this.policyId !== '') {
      if (this.policyId.startsWith('osdu/instance/')) {
        const instancePolicyId = this.policyId.replace('osdu/instance/','')
        console.log("instance query")
        this.noPolicyMessage = "Instance policies not currently supported " + instancePolicyId
        this.api.getInstancePolicy(this.authService.session.dataPartionId, instancePolicyId).subscribe((result) => {
          this.policy = result
        }, error => {
          this.policy = null
          if (error.status == 404) {
            this.noPolicyMessage = `No policy "${this.policyId}" in instance. Try checking in partition.`
          } else {
            this.noPolicyMessage = `Unable to retrieve policy "${this.policyId}" in instance. Policy Service returned error code ${error.status}`
          }
        })
      } else {
        this.api.getPolicy(this.authService.session.dataPartionId, this.policyId).subscribe((result) => {
          this.policy = result
        }, error => {
          this.policy = null
          if (error.status == 404) {
            this.noPolicyMessage = `No policy "${this.policyId}" in data partition "${this.authService.session.dataPartionId}".`
          } else {
            this.noPolicyMessage = `Unable to retrieve policy "${this.policyId}" in data partition "${this.authService.session.dataPartionId}". Policy Service returned error code ${error.status}`
          }
        })
      }
    }
  }

  evaluateQuery() {
    console.log("running evaluate")
    this.api.evaluateQuery(this.policyId, this.evaluateForm.value.query!, this.evaluateForm.value.includeAuth!).subscribe((result) => {
      //this.policy = JSON.stringify(result)
      console.log(JSON.stringify(result))
      const errors = result.result.records[0].errors
      if (errors.length == 0) {
        console.log("No errors")
        alert("Evalation OKAY")
      } else {
        console.log("Errors found: " + JSON.stringify(result))
        alert(JSON.stringify(result))
      }

    }, error => {
        if (error.status == 404) {
          this.noPolicyMessage = `No policy "${this.policyId}" in data partition "${this.authService.session.dataPartionId}".`
        } else if (error.status == 406) {
          this.noPolicyMessage = `Policy "${this.policyId}" in data partition "${this.authService.session.dataPartionId}".`
        } else {
          this.noPolicyMessage = `Unable to retrieve policy "${this.policyId}" in data partition "${this.authService.session.dataPartionId}". Policy Service returned error code ${error.status}`
        }
    })
  }

    /*
  editPolicy(event: any) {
  }

  deletePolicy(event: any) {
  }
    */

  getFilename(fullPath: string) {
    return fullPath.replace(/^.*[\\\/]/, '');
  }
}
