import { AfterViewInit, Component, OnInit, ViewChild } from '@angular/core';
import { MatTableDataSource } from '@angular/material/table';
import { SelectionModel } from '@angular/cdk/collections';
import { MatPaginator } from '@angular/material/paginator';
import { FormBuilder } from '@angular/forms';
import { Router } from '@angular/router';
import { startWith, withLatestFrom, map, of } from 'rxjs';
import { AuthService, AuthSession } from '../auth.service'
import { Policy, PolicyService } from '../policy.service'
import { TestBed } from '@angular/core/testing';



@Component({
  selector: 'app-policies',
  templateUrl: './policies.component.html',
  styleUrls: ['./policies.component.scss']
})
export class PoliciesComponent implements OnInit, AfterViewInit {
  @ViewChild(MatPaginator) paginator!: MatPaginator;

  public policies: null | Policy[] = null
  public displayedColumns = [
    // 'select',
    'id',
    'name',
  ];
  public tableData = new MatTableDataSource<Policy>([]);
  public initialSelection = [];
  public allowMultiSelect = false;
  public selection = new SelectionModel<Policy>(this.allowMultiSelect, this.initialSelection);
  public searchFormGroup = this.fb.group({
    filter: [''],
  });

  constructor(
    private api: PolicyService,
    private fb: FormBuilder,
    public authService: AuthService,
    private router: Router,
  ) { }

  ngOnInit(): void {
    this.api.getPolicies().subscribe((policies) => {
      this.policies = policies
      if (this.policies != null) {
        this.tableData.data = this.policies.map((policy) => {
          return {
            ...policy,
            lookupSupported: policy.id.startsWith('osdu/partition') || policy.id.startsWith('osdu/instance'),
          }
        })
      }
    })

    this.searchFormGroup.controls.filter.valueChanges.subscribe((filter) => {
      this.tableData.filter = filter ?? ''
    })
  }

  lookupSupported(policy: Policy): boolean {
    return policy.id.startsWith('osdu/partition') || policy.id.startsWith('osdu/instance')
  }

  ngAfterViewInit(): void {
    this.tableData.paginator = this.paginator
  }

  review(event: any) {
    event.stopPropagation()

    const item = this.selection.selected[0]
    const id = item.id

    if (id.startsWith('osdu/partition') || id.startsWith('osdu/instance')) {
      const shortId = id.substring(id.lastIndexOf('/') + 1)
      this.router.navigate(['/lookup'], { queryParams: { policyId: shortId }})
    } else {
      alert("Not supported on instance policies")
    }
  }

  edit(event: any) {
    event.stopPropagation()

    const item = this.selection.selected[0]
    const id = item.id

    if (id.startsWith('osdu/partition') || id.startsWith('osdu/instance')) {
      this.router.navigate(['/lookup'], { queryParams: { policyId: id }})
    } else {
      alert("Not supported on instance policies")
    }
  }

  delete(event: any) {
    event.stopPropagation()

    const item = this.selection.selected[0]
    const id = item.id

    this.router.navigate(['/lookup'], { queryParams: { policyId: id }})
  }
}
